@@ -1,4 +1,7 @@
 package Git::Repository;
+{
+  $Git::Repository::VERSION = '1.310';
+}
 
 use warnings;
 use strict;
@@ -11,8 +14,6 @@ use Scalar::Util qw( looks_like_number );
 
 use Git::Repository::Command;
 
-our $VERSION = '1.29';
-
 # a few simple accessors
 for my $attr (qw( git_dir work_tree options )) {
     no strict 'refs';
@@ -21,12 +22,10 @@ for my $attr (qw( git_dir work_tree options )) {
 
 # backward compatible aliases
 sub repo_path {
-    carp "repo_path() is obsolete, please use git_dir() instead";
-    goto &git_dir;
+    croak "repo_path() is obsolete, please use git_dir() instead";
 }
 sub wc_path {
-    carp "wc_path() is obsolete, please use work_tree() instead";
-    goto &work_tree;
+    croak "wc_path() is obsolete, please use work_tree() instead";
 }
 
 # helper function
@@ -67,16 +66,27 @@ sub new {
     my $self = bless {}, $class;
 
     # take out the option hash
-    my %arg = grep { !( ref eq 'HASH' ? $self->{options} ||= $_ : 0 ) } @arg;
-    my $options = $self->{options} ||= {};
+    my ( $options, %arg );
+    {
+        my @o;
+        %arg = grep !( ref eq 'HASH' ? push @o, $_ : 0 ), @arg;
+        croak "Too many option hashes given: @o" if @o > 1;
+        $options = $self->{options} = shift @o || {};
+    }
 
-    # ignore 'input' option during object creation
+    # ignore 'input' and 'fatal' options during object creation
     my $input = delete $options->{input};
+    my $fatal = delete $options->{fatal};
+
+    # die if deprecated parameters are given
+    croak "repository is obsolete, please use git_dir instead"
+        if defined delete $arg{repository};
+    croak "working_copy is obsolete, please use work_tree instead"
+        if defined delete $arg{working_copy};
 
     # setup default options
-    # accept older for backward compatibility
-    my ($git_dir) = grep {defined} delete @arg{qw( git_dir repository )};
-    my ($work_tree) = grep {defined} delete @arg{qw( work_tree working_copy )};
+    my $git_dir   = delete $arg{git_dir};
+    my $work_tree = delete $arg{work_tree};
 
     croak "Unknown parameters: @{[keys %arg]}" if keys %arg;
 
@@ -150,32 +160,14 @@ sub new {
 
     # put back the ignored option
     $options->{input} = $input if defined $input;
+    $options->{fatal} = $fatal if defined $fatal;
 
     return $self;
 }
 
+# create() is now fully deprecated
 sub create {
-    my ( $class, @args ) = @_;
-    my @output = $class->run(@args);
-    my $gitdir;
-
-    # create() is now deprecated
-    carp "create() is deprecated, please use run() instead";
-
-    # git init or clone until v1.7.1 (inclusive)
-    if ( $output[0] =~ /^(?:Reinitialized existing|Initialized empty) Git repository in (.*)/ ) {
-        $gitdir = $1;
-    }
-
-    # git clone after v1.7.1
-    elsif ( $output[0] =~ /Cloning into (bare repository )?(.*)\.\.\./ ) {
-        $gitdir = $1 ? $2 : File::Spec->catdir( $2, '.git' );
-    }
-
-    # some other command (no git repository created)
-    else {return}
-
-    return $class->new( git_dir => $gitdir, grep { ref eq 'HASH' } @args );
+    croak "create() is deprecated, see Git::Repository::Tutorial for better alternatives";
 }
 
 #
@@ -194,16 +186,17 @@ sub run {
     my ( $self, @cmd ) = @_;
 
     # split the args to get the optional callbacks
-    my @c;
-    @cmd = grep { ref eq 'CODE' ? !push @c, $_ : 1 } @cmd;
+    my @cb;
+    @cmd = grep { ref eq 'CODE' ? !push @cb, $_ : 1 } @cmd;
+
+    local $Carp::CarpLevel = 1;
 
     # run the command (pass the instance if called as an instance method)
     my $command
         = Git::Repository::Command->new( ref $self ? $self : (), @cmd );
 
     # return the output or die
-    local $Carp::CarpLevel = 1;
-    return $command->final_output(@c);
+    return $command->final_output(@cb);
 }
 
 #
@@ -294,12 +287,20 @@ sub version_ge {
 
 1;
 
+# ABSTRACT: Perl interface to Git repositories
+
+
 __END__
+=pod
 
 =head1 NAME
 
 Git::Repository - Perl interface to Git repositories
 
+=head1 VERSION
+
+version 1.310
+
 =head1 SYNOPSIS
 
     use Git::Repository;
@@ -321,12 +322,16 @@ Git::Repository - Perl interface to Git repositories
     Git::Repository->run( clone => $url, $dir, ... );
     $r = Git::Repository->new( work_tree => $dir );
 
-    # run commands
-    # - get the full output (no errput)
-    $output = $r->run(@cmd);
+    # provide an option hash for Git::Repository::Command
+    # (see Git::Repository::Command for all available options)
+    $r = Git::Repository->new( ..., \%options );
 
-    # - get the full output as a list of lines (no errput)
-    @output = $r->run(@cmd);
+    # run commands
+    # - get the full output (no errput) passing options for this command only
+    $output = $r->run( @cmd, \%options );
+
+    # - get the full output as a list of lines (no errput), with options
+    @output = $r->run( @cmd, \%options );
 
     # - process the output with callbacks
     $output = $r->run( @cmd, sub {...} );
@@ -334,7 +339,7 @@ Git::Repository - Perl interface to Git repositories
 
     # - obtain a Git::Repository::Command object
     #   (see Git::Repository::Command for details)
-    $cmd = $r->command(@cmd);
+    $cmd = $r->command( @cmd, \%options );
 
     # obtain version information
     my $version = $r->version();
@@ -371,11 +376,11 @@ any later version.
 
 See L<Git::Repository::Tutorial> for more code examples.
 
-=head1 CONSTRUCTORS
+=head1 CONSTRUCTOR
 
-There are two ways to create L<Git::Repository> objects:
+=head2 new
 
-=head2 new( %args, $options )
+    Git::Repository->new( %args, $options );
 
 Create a new L<Git::Repository> object, based on an existing Git repository.
 
@@ -413,20 +418,26 @@ corresponding L<Git::Repository> instance.
 
 So this:
 
-    my $options = {
-        git => '/path/to/some/other/git',
-        env => {
-            GIT_COMMITTER_EMAIL => 'book@cpan.org',
-            GIT_COMMITTER_NAME  => 'Philippe Bruhat (BooK)',
-        },
-    };
     my $r = Git::Repository->new(
+        # parameters
         work_tree => $dir,
-        $options
+        # options
+        {   git => '/path/to/some/other/git',
+            env => {
+                GIT_COMMITTER_EMAIL => 'book@cpan.org',
+                GIT_COMMITTER_NAME  => 'Philippe Bruhat (BooK)',
+            },
+        }
     );
 
 is equivalent to explicitly passing the option hash to each
-C<run()> or C<command()>.
+C<run()> or C<command()> call.
+The documentation for L<Git::Repository::Command> lists all
+available options.
+
+Note that Git::Repository and L<Git::Repository::Command> take
+great care in finding the option hash wherever it may be in C<@_>,
+and to merge multiple option hashes if more than one is provided.
 
 It probably makes no sense to set the C<input> option in C<new()>,
 but L<Git::Repository> won't stop you.
@@ -434,35 +445,29 @@ Note that on some systems, some git commands may close standard input
 on startup, which will cause a C<SIGPIPE>. L<Git::Repository::Command>
 will raise an exception.
 
-=head2 create( @cmd )
-
-B<The C<create()> method is deprecated, and will go away in the future.>
-
-Runs a repository initialization command (like C<init> or C<clone>) and
-returns a L<Git::Repository> object pointing to it. C<@cmd> may contain
-a hashref with options (see L<Git::Repository::Command>.
-
-Do not use the I<-q> option on such commands. C<create()> needs to parse
-their output to find the path to the repository.
-
-C<create()> also accepts a reference to an option hash which will be
-used to set up the returned L<Git::Repository> instance.
-
-Now that C<create()> is deprecated, instead of:
-
-    $r = Git::Repository->create( ... );
-
-simply do it in two steps:
+To create a Git repository and obtain a L<Git::Repository> object
+pointing to it, simply do it in two steps:
 
+    # run a clone or init command without an instance,
+    # using options like cwd
     Git::Repository->run( ... );
+    
+    # obtain a Git::Repository instance
+    # on the resulting repository
     $r = Git::Repository->new( ... );
 
-
 =head1 METHODS
 
+=for Pod::Coverage     create
+    repo_path
+    wc_path
+
 L<Git::Repository> supports the following methods:
 
-=head2 command( @cmd )
+=head2 command
+
+    Git::Repository->command( @cmd );
+    $r->command( @cmd );
 
 Runs the git sub-command and options, and returns a L<Git::Repository::Command>
 object pointing to the sub-process running the command.
@@ -470,7 +475,10 @@ object pointing to the sub-process running the command.
 As described in the L<Git::Repository::Command> documentation, C<@cmd>
 may also contain a hashref containing options for the command.
 
-=head2 run( @cmd )
+=head2 run
+
+    Git::Repository->run( @cmd );
+    $r->run( @cmd );
 
 Runs the command and returns the output as a string in scalar context,
 or as a list of lines in list context. Also accepts a hashref of options.
@@ -483,35 +491,28 @@ successively to each line of output. The line being processed is in C<$_>,
 but the coderef must still return the result string (like C<map>).
 
 If the git command printed anything on stderr, it will be printed as
-warnings. If the git sub-process exited with status C<128> (fatal error),
-or C<129> (usage message), C<run()> will C<die()>.
+warnings. For convenience, if the git sub-process exited with status
+C<128> (fatal error), or C<129> (usage message), C<run()> will C<die()>.
+The exit status values for which C<run()> dies can be modified using
+the C<fatal> option (see L<Git::Repository::Command> for details).
 
-=head2 git_dir()
+The exit status of the command that was just run is accessible as usual
+using C<<< $? >> 8 >>>. See L<perlvar> for details about C<$?>.
+
+=head2 git_dir
 
 Returns the repository path.
 
-=head2 repo_path()
-
-For backward compatibility with versions 1.06 and before, C<repo_path()>
-it provided as an alias to C<git_dir()>. It will be removed in a future
-version.
-
-=head2 work_tree()
+=head2 work_tree
 
 Returns the working copy path.
 Used as current working directory by L<Git::Repository::Command>.
 
-=head2 wc_path()
-
-For backward compatibility with versions 1.06 and before, C<wc_path()>
-it provided as an alias to C<work_tree()>. It will be removed in a future
-version.
-
-=head2 options()
+=head2 options
 
 Return the option hash that was passed to C<< Git::Repository->new() >>.
 
-=head2 version()
+=head2 version
 
 Return the version of git, as given by C<git --version>.
 
@@ -608,6 +609,13 @@ just prefix the fully qualified class name with a C<+>. For example:
 
 See L<Git::Repository::Plugin> about how to create a new plugin.
 
+=head1 ACKNOWLEDGEMENTS
+
+Thanks to Todd Rinalo, who wanted to add more methods to
+L<Git::Repository>, which made me look for a solution that would preserve
+the minimalism of L<Git::Repository>. The C<::Plugin> interface is what
+I came up with.
+
 =head1 OTHER PERL GIT WRAPPERS
 
 (This section was written in June 2010. The other Git wrappers have
@@ -630,10 +638,9 @@ Even though it works well for me and others, L<Git::Repository> has its
 own shortcomings: it I<is> a I<low-level interface to Git commands>,
 anything complex requires you to deal with input/output handles,
 it provides no high-level interface to generate actual Git commands
-or process the output of commands (but have a look at the plugins),
-it doesn't fully work under Win32 yet, etc. One the following modules
-may therefore be better suited for your needs, depending on what you're
-trying to achieve.
+or process the output of commands (but have a look at the plugins), etc.
+One the following modules may therefore be better suited for your needs,
+depending on what you're trying to achieve.
 
 =head2 Git.pm
 
@@ -646,10 +653,10 @@ It doesn't allow calling C<git init> or C<git clone>.
 The C<command_bidi_pipe> function especially has problems:
 L<http://kerneltrap.org/mailarchive/git/2008/10/24/3789584>
 
-
 =head2 Git::Class
 
-Depends on Moose, which seems an unnecessary dependency for a simple
+L<Git::Class>
+depends on Moose, which seems an unnecessary dependency for a simple
 wrapper around Git. The startup penalty could become significant for
 command-line tools.
 
@@ -658,23 +665,27 @@ Although it supports C<git init> and C<git clone>
 porcelain commands, and provides no way to control bidirectional commands
 (such as C<git commit-tree>).
 
-
 =head2 Git::Wrapper
 
-Doesn't support streams or bidirectional commands.
+L<Git::Wrapper>
+doesn't support streams or bidirectional commands.
 
+=head2 Git::Sub
 
-=head1 AUTHOR
+(This description was added for completeness in May 2013.)
 
-Philippe Bruhat (BooK), C<< <book at cpan.org> >>
+L<Git::Sub> appeared in 2013, as a set of Git-specific L<System::Sub>
+functions. It provide a nice set of C<git::> functions, and has some
+limitations (due to the way L<System::Sub> itself works) which don't
+impact most Git commands.
+
+L<Git::Sub> doesn't support working with streams.
 
 =head1 BUGS
 
 Since version 1.17, L<Git::Repository> delegates the actual command
-execution to L<System::Command>. Win32 support for that module is
-currently very bad (the test suite hangs in a few places).
-If you'd like better Win32 support for L<Git::Repository>, help me improve
-L<System::Command>!
+execution to L<System::Command>, which has better support for Win32
+since version 1.100.
 
 Please report any bugs or feature requests to C<bug-git-repository at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Git-Repository>.  I will be notified, and then you'll
@@ -686,7 +697,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Git::Repository
 
-
 You can also look for information at:
 
 =over 4
@@ -709,17 +719,23 @@ L<http://search.cpan.org/dist/Git-Repository>
 
 =back
 
+=head1 BUGS
 
-=head1 ACKNOWLEDGEMENTS
+Please report any bugs or feature requests on the bugtracker website
+http://rt.cpan.org/NoAuth/Bugs.html?Dist=Git-Repository or by email to
+bug-git-repository@rt.cpan.org.
 
-Thanks to Todd Rinalo, who wanted to add more methods to
-L<Git::Repository>, which made me look for a solution that would preserve
-the minimalism of L<Git::Repository>. The C<::Plugin> interface is what
-I came up with.
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=head1 AUTHOR
+
+Philippe Bruhat (BooK) <book@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright 2010-2012 Philippe Bruhat (BooK), all rights reserved.
+Copyright 2010-2014 Philippe Bruhat (BooK), all rights reserved.
 
 =head1 LICENSE
 
